@@ -1,3 +1,4 @@
+import json
 from urllib.parse import urlparse
 from collections import namedtuple
 from django.views.generic import TemplateView, View
@@ -6,7 +7,7 @@ from django.utils.safestring import mark_safe
 from django.urls import reverse, resolve, Resolver404
 from ajax_helpers.mixins import AjaxHelpers
 from ajax_helpers.templatetags.ajax_helpers import button_javascript
-from ajax_helpers.utils import random_string
+from ajax_helpers.utils import random_string, ajax_command
 from django.conf import settings
 
 
@@ -22,7 +23,7 @@ class MenuItemBadge:
         if self.format_function:
             self.format_function(self)
         if self.text:
-            return mark_safe(f' <sup><span class="badge badge-pill badge-{self.css_class}">{self.text}</span></sup>')
+            return mark_safe(f'&nbsp;<sup><span class="badge badge-pill badge-{self.css_class}">{self.text}</span></sup>')
         return ''
 
     def __str__(self):
@@ -58,22 +59,22 @@ class DividerItem(BaseMenuItem):
 
 
 class MenuItemDisplay:
-    def __init__(self, text=None, font_awesome=None, css_classes=None):
+    def __init__(self, text=None, font_awesome=None, css_classes=None, tooltip=None, attributes=None):
         self._css_classes = None
 
-        if isinstance(text, self.__class__):
-            self.text = text.text
-            self.font_awesome = font_awesome if font_awesome else text.font_awesome
-            self.css_classes = css_classes if css_classes else text.css_classes
-        elif isinstance(text, (tuple, list)):
+        if isinstance(text, (tuple, list)):
             params = {c: v for c, v in enumerate(text)}
             self.text = params.get(0)
             self.font_awesome = font_awesome if font_awesome else params.get(1)
             self.css_classes = css_classes if css_classes else params.get(2)
+            self.tooltip = tooltip if tooltip else params.get(3)
+            self._attributes = attributes if attributes else params.get(4)
         else:
             self.text = text
             self.font_awesome = font_awesome
             self.css_classes = css_classes
+            self.tooltip = tooltip
+            self._attributes = attributes
 
     def display(self):
         if self.font_awesome:
@@ -93,6 +94,9 @@ class MenuItemDisplay:
         else:
             self._css_classes = css
 
+    def attributes(self):
+        return MenuItem.attr(self._attributes, self.tooltip)
+
 
 class MenuItem(BaseMenuItem):
 
@@ -114,17 +118,26 @@ class MenuItem(BaseMenuItem):
     def menu(self, menu):
         self._menu = menu
         if menu.button_defaults and self.name in menu.button_defaults:
-            self.menu_display = MenuItemDisplay(menu.button_defaults[self.name])
+            self.menu_display = menu.button_defaults[self.name]
+            if not isinstance(self.menu_display, MenuItemDisplay):
+                self.menu_display = MenuItemDisplay(self.menu_display)
         if self.dropdown:
             self.dropdown.menu = menu
 
     def css(self):
         return ' '.join(self.menu_display.css_classes + (['disabled'] if self.disabled else []))
 
+    @staticmethod
+    def attr(attributes, tooltip):
+        attributes = {} if attributes is None else attributes
+        if tooltip:
+            attributes.update({'title': tooltip, 'data-tooltip': 'tooltip', 'data-placement': 'bottom'})
+        return attributes
+
     def __init__(self, url=None, menu_display=None, link_type=URL_NAME, css_classes=None, template=None,
                  badge=None, target=None, dropdown=None, show_caret=True, font_awesome=None, no_hover=False,
                  placement='bottom-start', url_args=None, url_kwargs=None, attributes=None,
-                 dropdown_kwargs=None, **kwargs):
+                 dropdown_kwargs=None, tooltip=None, **kwargs):
         super().__init__(**kwargs)
         self._resolved_url = None
         self.link_type = link_type
@@ -134,7 +147,7 @@ class MenuItem(BaseMenuItem):
                 url_args = split_url[1:]
                 url = split_url[0]
         self._href = self.raw_href(url, url_args, url_kwargs)
-        self._attributes = attributes
+        self._attributes = self.attr(attributes, tooltip)
         self.menu_config = {}
         if url is not None and link_type in self.RESOLVABLE_LINK_TYPES and self.resolved_url != 'invalid':
             view_class = getattr(self.resolved_url.func, 'view_class', None)
@@ -149,7 +162,10 @@ class MenuItem(BaseMenuItem):
                 else:
                     # noinspection PyTypeChecker
                     self.menu_config: dict = view_class.menu_config
-        self.menu_display = MenuItemDisplay(menu_display, font_awesome, css_classes)
+        if isinstance(menu_display, MenuItemDisplay):
+            self.menu_display = menu_display
+        else:
+            self.menu_display = MenuItemDisplay(menu_display, font_awesome, css_classes)
         self.kwargs = kwargs
         self.template = template
         self.target = target
@@ -177,8 +193,8 @@ class MenuItem(BaseMenuItem):
                 attributes.update(self.menu_config['attributes'])
             else:
                 attributes.update(self.external_function(self.menu_config['attributes']))
-        if self._attributes:
-            attributes.update(self._attributes)
+        attributes.update(self._attributes)
+        attributes.update(self.menu_display.attributes())
         if attributes:
             return mark_safe(' '.join([f'{k}="{v}"' for k, v in attributes.items()]))
         return ''
@@ -202,7 +218,7 @@ class MenuItem(BaseMenuItem):
     @property
     def active(self):
         if self.menu:
-            if self.menu.active:
+            if self.menu.active and self.resolved_url != 'invalid':
                 try:
                     url_name = self.resolved_url.url_name
                     if self.resolved_url.namespace:
@@ -211,9 +227,12 @@ class MenuItem(BaseMenuItem):
                         return True
                 except Resolver404:
                     return
-            else:
-                if self.link_type in self.RESOLVABLE_LINK_TYPES and self.menu.request.path == self._href:
-                    return True
+            elif self.menu.request is not None:
+                if self.link_type in self.RESOLVABLE_LINK_TYPES:
+                    if self.menu.compare_full_path:
+                        return self.menu.request.get_full_path() == self._href
+                    else:
+                        return self.menu.request.path == self._href
 
     @property
     def badge(self):
@@ -277,8 +296,8 @@ class HtmlMenu:
         'buttons': 'django_menus/button_menu.html',
     }
 
-    def __init__(self, request=None, template='base', menu_id=None,
-                 placement=None, no_hover=False, button_defaults=None, alignment=None):
+    def __init__(self, request=None, template='base', menu_id=None, default_link_type=MenuItem.URL_NAME,
+                 placement=None, no_hover=False, button_defaults=None, alignment=None, compare_full_path=False):
         self.menu_items = []
         self.button_defaults = getattr(settings, 'DJANGO_MENUS_BUTTON_DEFAULTS', {})
         if button_defaults is not None:
@@ -292,11 +311,14 @@ class HtmlMenu:
         self.alignment = alignment
         self.fixed_id = menu_id
         self.id = None
+        self.compare_full_path = compare_full_path
+        self.default_link_type = default_link_type
 
     def visible_items(self):
         return [i for i in self.menu_items if i.visible]
 
-    def add_item(self, url_name=None, text=None,  link_type=MenuItem.URL_NAME, **kwargs):
+    def add_item(self, url_name=None, text=None,  link_type=None, **kwargs):
+        link_type = link_type if link_type is not None else self.default_link_type
         self.menu_items.append(MenuItem(url=url_name, menu_display=text, menu=self, link_type=link_type, **kwargs))
 
     def add_items(self, *args):
@@ -388,35 +410,58 @@ class AjaxMenuTabs(AjaxMenuTemplateView):
     additional_content = []
 
     def create_ajax_commands(self, context):
-        for c in self.ajax_commands:
+        self.add_command('clear_timers', store='tab')
+        for c in self.ajax_response_commands:
             if c.type == self.TEMPLATE_CONTENT:
                 html = context[c.name]
             else:
                 html = self.menus[c.name].render()
             self.add_command('html', selector='#' + c.name, html=html)
 
+    def __init__(self, *args, **kwargs):
+        self.ajax_response_commands = None
+        super().__init__(*args, **kwargs)
+
+    def set_response_commands(self):
+        if not self.ajax_response_commands:
+            self.ajax_response_commands = self._ajax_commands + [self.AjaxCommand(*a) for a in self.additional_content]
+
+    def tab_response(self):
+        self.set_response_commands()
+        context = self.get_context_data(**self.kwargs)
+        self.create_ajax_commands(context)
+        response = self.command_response()
+        response['Cache-Control'] = 'No-Cache,No-Store'
+        return response
+
     def get(self, request, *args, **kwargs):
-        self.ajax_commands = self._ajax_commands + [self.AjaxCommand(*a) for a in self.additional_content]
+        self.set_response_commands()
         if request.is_ajax():
-            context = self.get_context_data(**kwargs)
-            self.create_ajax_commands(context)
-            response = self.command_response()
-            response['Cache-Control'] = 'No-Cache,No-Store'
-            return response
+            return self.tab_response()
         return super().get(request, *args, **kwargs)
 
     def main_context(self, **kwargs):
         return {}
 
+    def get_tab_commands(self):
+        pass
+
     def tab_context(self, **kwargs):
-        return {}
+        tab_commands = self.get_tab_commands()
+        context = {}
+        if tab_commands:
+            command = ajax_command('onload', commands=tab_commands)
+            context['tabs_script'] = mark_safe(
+                f'<script>ajax_helpers.process_commands([{json.dumps(command)}])</script>'
+            )
+        return context
 
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
         if not self.request.is_ajax():
             context.update(self.main_context())
         context.update(self.tab_context())
-        for c in self.ajax_commands:
+        for c in self.ajax_response_commands:
             if c.type == self.TEMPLATE_CONTENT:
                 context[c.name] = mark_safe(render_to_string(getattr(self, c.name), context=context))
         return context
